import datetime
from django.conf import settings
from django.utils.timezone import now

from .models import CanIHitIt


def canihitit(request, object_type, object_id):
    return _canihitit(
        request.session.session_key or request.META['REMOTE_ADDR'] or '',
        object_type,
        object_id,
        request.META['HTTP_USER_AGENT']
    )


def _canihitit(session_key, object_type, object_id, user_agent=None):
    seconds = getattr(settings, 'CAN_I_HIT_IT_SECONDS', 300)
    cutoff = now() - datetime.timedelta(seconds=seconds)

    # ignore a few known bots. TODO: improve this
    if user_agent:
        user_agent = user_agent.lower()
        bots = 'baiduspider,bingbot,duckduckbot,exabot,facebookexternalhit,' \
               'facebot,googlebot,ia_archiver,sogou.com,yahoo! slurp,yandexbot'
        for b in bots.split(','):
            if b in user_agent:
                return False

    if CanIHitIt \
        .objects \
        .filter(object_type=object_type) \
        .filter(object_id=object_id) \
        .filter(session_key=session_key) \
        .filter(created__gte=cutoff) \
        .count() > 0:
        return False

    cihi = CanIHitIt(
        object_type=object_type,
        object_id=object_id,
        session_key=session_key
    )
    cihi.save()

    return True



from django.contrib import admin
from .models import CanIHitIt


class CanIHitItAdmin(admin.ModelAdmin):
    list_display = ('object_type', 'object_id', 'session_key', 'created')
    list_filter = ('object_type', 'session_key')


admin.site.register(CanIHitIt, CanIHitItAdmin)



from django.conf import settings
from django.db import models


class CanIHitIt(models.Model):
    object_type = models.CharField(max_length=32, db_index=True)
    object_id = models.IntegerField(db_index=True)
    session_key = models.CharField(max_length=40, db_index=True)
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return '%s: %s#%s' % (self.id, self.object_type, self.object_id)



# Generated by Django 2.1.4 on 2019-02-11 09:19

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CanIHitIt',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_type', models.CharField(db_index=True, max_length=32)),
                ('object_id', models.IntegerField(db_index=True)),
                ('session_key', models.CharField(db_index=True, max_length=40)),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]
